"""This package implements a series of matching algorithms.

All algorithms should implement :class:`.AbstractMatchingAlgorithm`.
"""

from py_school_match.algorithms.da_stb import DASTB

from py_school_match.algorithms.da_mtb import DAMTB

from py_school_match.algorithms.sic import SIC
from py_school_match.algorithms.msic import MSIC
from py_school_match.algorithms.nsic import NSIC
from py_school_match.algorithms.mnsic import MNSIC

from py_school_match.algorithms.ttc import TTC

from py_school_match.algorithms.da import DA
from py_school_match.algorithms.pi import PI
