"""This package implements a series of entities which are
necessary to define any matching problem."""

from py_school_match.entities.characteristic import Characteristic
from py_school_match.entities.criteria import Criteria
from py_school_match.entities.ruleset import RuleSet
from py_school_match.entities.rule import Rule

from py_school_match.entities.school import School
from py_school_match.entities.slot import Slot

from py_school_match.entities.social_planner import SocialPlanner

from py_school_match.entities.student import Student
from py_school_match.entities.student_queue import StudentQueue
