"""This package implements some util methods.
"""

from py_school_match.util.display_info import get_positions_stat, display_assignation, display_positions_from_planner, display_positions
from py_school_match.util.display_info import display_assignation_from_planner, save_positions, save_data_from_planner

from py_school_match.util.file_util import gen_filepath, get_number_list_from_file

from py_school_match.util.stat import get_variance

from py_school_match.util.simulate_preferences import simulate_preferences
from py_school_match.util.simulate_characteristics import simulate_characteristics